diff -Nur openbsc.patched/include/openbsc/gsm_04_08.h openbsc.new/include/openbsc/gsm_04_08.h
--- openbsc.patched/include/openbsc/gsm_04_08.h	2009-05-23 17:32:15.000000000 +0200
+++ openbsc.new/include/openbsc/gsm_04_08.h	2009-06-01 11:09:21.000000000 +0200
@@ -528,6 +528,8 @@
 
 int gsm48_send_rr_release(struct gsm_lchan *lchan);
 
+void release_loc_updating_req(struct gsm_lchan *lchan);
+
 /* convert a ASCII phone number to call-control BCD */
 int encode_bcd_number(u_int8_t *bcd_lv, u_int8_t max_len,
 		      u_int8_t type, const char *input);
diff -Nur openbsc.patched/include/openbsc/gsm_data.h openbsc.new/include/openbsc/gsm_data.h
--- openbsc.patched/include/openbsc/gsm_data.h	2009-05-31 12:59:24.000000000 +0200
+++ openbsc.new/include/openbsc/gsm_data.h	2009-06-01 13:23:48.000000000 +0200
@@ -40,10 +40,18 @@
 #define LCHAN_RELEASE_TIMEOUT 4, 0
 #define use_lchan(lchan) \
 	do {	lchan->use_count++; \
+		DEBUGP(DCC, "lchan (bts=%d,trx=%d,ts=%d,ch=%d) increases usage to: %d\n", \
+			lchan->ts->trx->bts->nr, lchan->ts->trx->nr, lchan->ts->nr, \
+			lchan->nr, lchan->use_count); \
 		bsc_schedule_timer(&lchan->release_timer, LCHAN_RELEASE_TIMEOUT); } while(0);
 
 #define put_lchan(lchan) \
-	do { lchan->use_count--; } while(0);
+	do { lchan->use_count--; \
+		DEBUGP(DCC, "lchan (bts=%d,trx=%d,ts=%d,ch=%d) decreases usage to: %d\n", \
+			lchan->ts->trx->bts->nr, lchan->ts->trx->nr, lchan->ts->nr, \
+			lchan->nr, lchan->use_count); \
+	} while(0);
+
 
 /* communications link with a BTS */
 struct gsm_bts_link {
diff -Nur openbsc.patched/src/abis_rsl.c openbsc.new/src/abis_rsl.c
--- openbsc.patched/src/abis_rsl.c	2009-05-31 12:59:25.000000000 +0200
+++ openbsc.new/src/abis_rsl.c	2009-06-01 13:42:11.000000000 +0200
@@ -680,12 +680,14 @@
 		if (TLVP_PRESENT(&tp, RSL_IE_CAUSE) &&
 		    TLVP_LEN(&tp, RSL_IE_CAUSE) >= 1 &&
 		    *TLVP_VAL(&tp, RSL_IE_CAUSE) == 0x18) {
-			DEBUGPC(DRSL, "IGNORING\n");
-			return 0;
+			if (msg->lchan->use_count > 0) {
+				DEBUGPC(DRSL, "Cause 0x18 IGNORING, lchan in use! (%d times)\n", msg->lchan->use_count);
+				return 0;
+			}
 		}
 	}
 
-	DEBUGPC(DRSL, "\n");
+	DEBUGPC(DRSL, "RELEASING.\n");
 
 	/* FIXME: only free it after channel release ACK */
 	return rsl_chan_release(msg->lchan);
diff -Nur openbsc.patched/src/chan_alloc.c openbsc.new/src/chan_alloc.c
--- openbsc.patched/src/chan_alloc.c	2009-05-23 17:32:17.000000000 +0200
+++ openbsc.new/src/chan_alloc.c	2009-06-01 11:21:44.000000000 +0200
@@ -193,6 +193,10 @@
 		lchan->subscr = 0;
 	}
 
+	/* Free usage for location operation */
+	if (lchan->loc_operation)
+		release_loc_updating_req(lchan);
+
 	/* We might kill an active channel... FIXME: call cancellations */
 	if (lchan->use_count != 0) {
 		lchan->use_count = 0;
diff -Nur openbsc.patched/src/gsm_04_08.c openbsc.new/src/gsm_04_08.c
--- openbsc.patched/src/gsm_04_08.c	2009-06-01 10:50:09.000000000 +0200
+++ openbsc.new/src/gsm_04_08.c	2009-06-01 13:36:42.000000000 +0200
@@ -161,7 +161,7 @@
 	return subscriber->authorized;
 }
 
-static void release_loc_updating_req(struct gsm_lchan *lchan)
+void release_loc_updating_req(struct gsm_lchan *lchan)
 {
 	if (!lchan->loc_operation)
 		return;
@@ -485,7 +485,6 @@
 	/*
 	 * Rogue messages could trick us but so is life
 	 */
-	put_lchan(lchan);
 
 	switch (mi_type) {
 	case GSM_MI_TYPE_IMSI:
@@ -575,7 +574,6 @@
 	switch (mi_type) {
 	case GSM_MI_TYPE_IMSI:
 		/* we always want the IMEI, too */
-		use_lchan(lchan);
 		rc = mm_tx_identity_req(lchan, GSM_MI_TYPE_IMEI);
 		lchan->loc_operation->waiting_for_imei = 1;
 
@@ -584,7 +582,6 @@
 		break;
 	case GSM_MI_TYPE_TMSI:
 		/* we always want the IMEI, too */
-		use_lchan(lchan);
 		rc = mm_tx_identity_req(lchan, GSM_MI_TYPE_IMEI);
 		lchan->loc_operation->waiting_for_imei = 1;
 
@@ -592,7 +589,6 @@
 		subscr = subscr_get_by_tmsi(mi_string);
 		if (!subscr) {
 			/* send IDENTITY REQUEST message to get IMSI */
-			use_lchan(lchan);
 			rc = mm_tx_identity_req(lchan, GSM_MI_TYPE_IMSI);
 			lchan->loc_operation->waiting_for_imsi = 1;
 		}
@@ -857,8 +855,6 @@
 	} else
 		DEBUGP(DMM, "Unknown Subscriber ?!?\n");
 
-	put_lchan(msg->lchan);
-
 	return 0;
 }
 
@@ -933,7 +929,6 @@
 	if (!subscr) {
 		DEBUGP(DRR, "<- Can't find any subscriber for this ID\n");
 		/* FIXME: request id? close channel? */
-		return -EINVAL;
 	}
 	DEBUGP(DRR, "<- Channel was requested by %s\n",
 		subscr->name ? subscr->name : subscr->imsi);
